﻿using System;
using System.Windows.Forms;
using System.Threading;
using System.IO.Ports;





namespace WS2812_USB_UART
{
    internal partial class FormMain : Form
    {

        WS_UART wsu;
        SerialPort sp = new SerialPort();
        IntervalTimer it;

        public FormMain()
        {
            InitializeComponent();

            hS_Helligkeit.Maximum += hS_Helligkeit.LargeChange - 1;
            hS_Helligkeit.Value = 100;

            hS_S.Maximum += hS_S.LargeChange - 1;
            hS_S.Value = 100;


            wsu = new WS_UART(USB_Device.S840_PL2303HXA, 120);    // Device und LED's
            RGB_Array = (byte[])wsu.RGB_Array.Clone();


            wsu.SetupSerialPort(sp);

            cBB_COM.Items.AddRange(SerialPort.GetPortNames());
            if (cBB_COM.Items.Count > 0)
            {
                cB_EinAus.Enabled = true;
                cBB_COM.SelectedIndex = cBB_COM.Items.Count - 1;
            }
            else
            {
                cB_EinAus.Enabled = false;
                MessageBox.Show("Keine COM-Ports");
            }


            it = new IntervalTimer(10000, IGJitter.Low); // 10000µs = 100Hz
            it.IntervalElapsed += new IntervalElapsedEventHandler(it_IntervalElapsed);
        }


        void it_IntervalElapsed(object sender, EventArgs e)
        {
            sp.Write(wsu.UartArray, 0, wsu.UartArray.Length);

            BallFadeEffekt();

            GammaAdjustment.Adjust(wsu.RGB_Array, RGB_Array);

            //wsu.RGB_Array[1] = 1 << 0 | 1 << 1 | 0 << 2 | 1 << 3 | 1 << 4 | 0 << 5 | 0 << 6 | 0 << 7;
            //wsu.RGB_Array[1] = 0;
            wsu.RgbArrayToWsUartArray();
        }


        byte[] RGB_Array;
        HSVColor HSVBall = new HSVColor(0f, 1f, 1f);
        RGBColor RGBBall = new RGBColor();
        int BallPos;

        void BallFadeEffekt()
        {
            byte AlterWert, NeuerWert;

            for (int i = 0; i < wsu.LedCount * 3; i++)
            {
                AlterWert = RGB_Array[i];
                NeuerWert = RGB_Array[i] = unchecked((byte)(AlterWert - 5));
                
                if (NeuerWert > AlterWert)
                    RGB_Array[i] = 0;
            }

            HSVBall.H = (HSVBall.H + 5f) % 360f;
            RGBBall = HSV2RGB.Convert(HSVBall);
            RGB_Array[BallPos * 3 + 0] = RGBBall.ByteR;
            RGB_Array[BallPos * 3 + 1] = RGBBall.ByteG;
            RGB_Array[BallPos * 3 + 2] = RGBBall.ByteB;

            if (++BallPos >= wsu.LedCount) { BallPos = 0; }

        }


        private void hS_Helligkeit_ValueChanged(object sender, EventArgs e)
        {
            HScrollBar hs = (HScrollBar)sender;

            GammaAdjustment.Helligkeit = hs.Value;
        }


        private void hS_S_ValueChanged(object sender, EventArgs e)
        {
            HScrollBar hs = (HScrollBar)sender;

            HSVBall.S = hs.Value / 100f;
        }


        private void cBB_COM_SelectedIndexChanged(object sender, EventArgs e)
        {
            ComboBox cb = (ComboBox)sender;

            sp.PortName = cb.Text;
        }


        private void cB_EinAus_CheckedChanged(object sender, EventArgs e)
        {
            CheckBox cb = (CheckBox)sender;

            if (cb.Checked)
            {
                cBB_COM.Enabled = false;
                try { sp.Open(); }
                catch
                {
                    // Beim ersten mal weigert sich das Device manchmal.
                    Thread.Sleep(5000);
                    try { sp.Open(); }
                    catch (Exception eOpen)
                    {
                        MessageBox.Show(eOpen.Message);

                        cb.Checked = false;

                        return;
                    }
                }
                
                it.Start();
            }
            else
            {
                if (sp.IsOpen)
                {
                    it.Stop();
                    sp.Close();
                }

                cBB_COM.Enabled = true;
            }
        }


        private void FormMain_FormClosing(object sender, FormClosingEventArgs e)
        {
            it.Stop();
            sp.Dispose();
        }


    }

}
